# Copyright 2023 Luminary Cloud, Inc. All Rights Reserved.
import logging
from typing import Any, Callable

import grpc
from grpc import (
    ClientCallDetails,
    UnaryUnaryClientInterceptor,
)
from grpc._interceptor import _ClientCallDetails

logger = logging.getLogger(__name__)

from importlib.metadata import version

try:
    _lc_sdk_version = "v" + version("luminarycloud")
except:
    _lc_sdk_version = "unknown"


class LoggingInterceptor(UnaryUnaryClientInterceptor):
    """
    A client interceptor that logs gRPC method calls and appends the SDK version
    to the request metadata.
    """

    def intercept_unary_unary(
        self,
        continuation: Callable[[ClientCallDetails, Any], grpc.Call],
        client_call_details: ClientCallDetails,
        request: Any,
    ) -> grpc.Call:

        metadata = []
        if client_call_details.metadata is not None:
            metadata = list(client_call_details.metadata)
        # will look like: x-client-version: python-sdk-v0.1.0
        metadata.append(("x-client-version", f"python-sdk-{_lc_sdk_version}"))

        client_call_details = _ClientCallDetails(
            client_call_details.method,
            client_call_details.timeout,
            metadata,
            client_call_details.credentials,
            client_call_details.wait_for_ready,
            client_call_details.compression,
        )

        logger.debug(
            f"[SDK {_lc_sdk_version}] Sending unary-unary gRPC request: {client_call_details.method}."
        )
        try:
            call = continuation(client_call_details, request)
            call.result()
        except grpc.RpcError:
            logger.debug("Error occurred during gRPC request", exc_info=True)
            raise
        return call
